@@ -1,11 +1,8 @@
 #
 # Copyright (C) 2006, 2007, 2008 Loic Dachary <loic@dachary.org>
-# Copyright (C) 2004, 2005, 2006 Mekensleep
-#
-# Mekensleep
-# 24 rue vieille du temple
-# 75004 Paris
-#       licensing@mekensleep.com
+# Copyright (C)       2008, 2009 Bradley M. Kuhn <bkuhn@ebb.org>
+# Copyright (C) 2004, 2005, 2006 Mekensleep <licensing@mekensleep.com>
+#                                24 rue vieille du temple, 75004 Paris
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -23,6 +20,7 @@
 #
 # Authors:
 #  Loic Dachary <loic@gnu.org>
+#  Bradley M. Kuhn <bkuhn@ebb.org>
 #
 from math import ceil
 from types import StringType
@@ -35,7 +33,7 @@ def tournament_seconds():
 shuffler = random
 
 from pokerengine.pokergame import PokerGameServer
-from pokerengine.pokerengineconfig import Config
+from pokerengine import pokerprizes
 
 TOURNAMENT_STATE_ANNOUNCED = "announced"
 TOURNAMENT_STATE_REGISTERING = "registering"
@@ -217,6 +215,12 @@ class PokerTournament:
         
         self.callback_new_state = lambda tournament, old_state, new_state: True
         self.callback_create_game = lambda tournament: PokerGameServer("poker.%s.xml", tournament.dirs)
+        # I think callback_game_filled() is a misnomer because it's not
+        # about the table being "filled" (i.e., the table could have less
+        # than the max seated at it).  What really happens is that the
+        # callback_game_filled() is made when the table is deemed to have
+        # the number of players at it the tourney manager has decided
+        # belong there (which may or may not be "filled").
         self.callback_game_filled = lambda tournament, game: True
         self.callback_destroy_game = lambda tournament, game: True
         self.callback_move_player = lambda tournament, from_game_id, to_game_id, serial: self.movePlayer(from_game_id, to_game_id, serial)
@@ -226,12 +230,11 @@ class PokerTournament:
         self.updateRegistering()
 
     def loadPayouts(self):
-        config = Config(self.dirs)
-        config.load("poker.payouts.xml")
-        self.payouts = []
-        for node in config.header.xpathEval("/payouts/payout"):
-            properties = config.headerNodeProperties(node)
-            self.payouts.append(( int(properties['max']), map(lambda percent: float(percent) / 100, node.content.split())))
+        if self.sit_n_go == 'y':
+            player_count = self.players_quota
+        else:
+            player_count = self.registered
+        self.prizes_object =  pokerprizes.__dict__['PokerPrizes' + self.prizes_specs.capitalize()](buy_in_amount = self.buy_in, player_count = player_count, guarantee_amount = self.prize_min, config_dirs = self.dirs)
 
     def message(self, message):
         print self.prefix + "[PokerTournament %s] " % self.name + message
@@ -337,6 +340,7 @@ class PokerTournament:
             self.createGames()
             self.can_register = False
         elif self.state == TOURNAMENT_STATE_REGISTERING and state == TOURNAMENT_STATE_CANCELED:
+            self.can_register = False
             self.cancel()
             self.finish_time = tournament_seconds()
         elif ( self.state in ( TOURNAMENT_STATE_RUNNING, TOURNAMENT_STATE_BREAK_WAIT ) and
@@ -366,6 +370,9 @@ class PokerTournament:
         if self.can_register:
             self.players.append(serial)
             self.registered += 1
+            if self.sit_n_go != 'y':
+                self.prizes_object.addPlayer()
+                self.rank2prize = None
             if self.state == TOURNAMENT_STATE_REGISTERING:
                 self.updateRunning()
             elif self.state == TOURNAMENT_STATE_RUNNING:
@@ -378,6 +385,9 @@ class PokerTournament:
         if self.state == TOURNAMENT_STATE_REGISTERING:
             self.players.remove(serial)
             self.registered -= 1
+            if self.sit_n_go != 'y':
+                self.prizes_object.removePlayer()
+                self.rank2prize = None
             return True
         else:
             return False
@@ -442,9 +452,16 @@ class PokerTournament:
                 game.autoBlindAnte(player)
                 
             self.games.append(game)
+        self.id2game = dict(zip([ game.id for game in self.games ], self.games))
+        # Next, need to call balance games, because the table assignment
+        # algorithm above does not account for scenarios where the last
+        # few people end up a table too small.
+        self.balanceGames()
+        # Next, we can now notify via callback that all the games in
+        # self.games have been "filled".
+        for game in self.games:
             self.callback_game_filled(self, game)
             game.close()
-        self.id2game = dict(zip([ game.id for game in self.games ], self.games))
 
     def endTurn(self, game_id):
         game = self.id2game[game_id]
@@ -511,60 +528,6 @@ class PokerTournament:
         return len(to_equalize) > 0
 
     def prizes(self):
-        if self.can_register:
-            return None
         if not self.rank2prize:
-            if self.prizes_specs == "algorithm":
-                self.rank2prize = self.prizesAlgorithm()
-            elif self.prizes_specs == "table":
-                self.rank2prize = self.prizesTable()
+            self.rank2prize = self.prizes_object.getPrizes()
         return self.rank2prize
-
-    def prizesAlgorithm(self):
-        buy_in = self.buy_in
-        candidates_count = self.registered
-        if candidates_count < 5:
-            winners = 1
-        elif candidates_count < 10:
-            winners = 2
-        elif candidates_count < 20:
-            winners = 3
-        elif candidates_count < 30:
-            winners = 4
-        elif candidates_count < 40:
-            winners = 6
-        elif candidates_count < 50:
-            winners = int(candidates_count * 0.2)
-        elif candidates_count < 200:
-            winners = int(candidates_count * 0.15)
-        else:
-            winners = int(candidates_count * 0.1)
-
-        prizes = []
-        prize_pool = max(self.prize_min, buy_in * candidates_count)
-        money_left = prize_pool
-        while winners > 0:
-            if money_left / winners < max(1, prize_pool / 100, int(buy_in * 2.5)):
-                prizes.extend([ money_left / winners ] * winners)
-                winners = 0
-            else:
-                money_left /= 2
-                winners -= 1
-                prizes.append(money_left)
-        rest = prize_pool - sum(prizes)
-        prizes[0] += rest
-        return prizes
-                
-    def prizesTable(self):
-        buy_in = self.buy_in
-        for ( maximum, payouts ) in self.payouts:
-            if self.registered <= maximum:
-                break
-
-        total = max(self.prize_min, self.registered * buy_in)
-        prizes = map(lambda percent: int(total * percent), payouts)
-        #
-        # What's left because of rounding errors goes to the tournament winner
-        #
-        prizes[0] += total - sum(prizes)
-        return prizes
